*-----------------------------------------------------------
* Title      : Maze Runner
* Written by : Tanner Trombley
* Date       : 22 March 2013
* Description: A simple maze traversal game. A random maze
*              is generated and the player uses the arrow 
*              keys to navigate from the top left to the 
*              bottom right.
*-----------------------------------------------------------
    ORG    $1000
    
WHITE   	equ $00ffffff
LIME    	equ $0000ff00
BLUE    	equ $00ff0000
BLACK   	equ $00000000
BACKGROUND    	equ $00800000

START:
    move.l  	#keyDownIRQ,$68     		Initializes IRQ vectors
    move.l	#mouseDownIRQ,$64 		-                  
    move.b	#60,d0			  	-
    move.w	#$0101,d1		  	-
    trap	#15				-
    
    move.b	#0,contin		 	Sets the continue variable equal to 0 
    bsr		display_title	  		displays the title screen
    
    repeat					repeats until continue is set in mouseDownIRQ
	nop				-
    until.b contin <eq> #1 do 		-

reset					    	*branches back here after a new game
	
    move.b  #60,d0				turns the mouseDownIRQ off
    move.w  #$0001,d1			-
    trap    #15					-
    
    bsr     seed_rand       			seeds random # generator
    bsr     load_maze				creates the maze in the window
    
    
    							
    move.b  #62,d0				Initialize IRQ vector for keyDown
    move.w  #$0201,d1				-
    trap    #15					-
        
    move.w  #$0101,d1				sets cursor position to (1,1)
    move.b  #11,d0				-
    trap    #15					-
    

    repeat					game loop. Runs until player wins
	nop				-
    until.b gmeovr <eq> #1 do	-	
    
    move.b	#62,d0				turns off the keyDownIRQ
    move.w	#$0001,d1			-
    trap	#15				-
    	
    move.b	#60,d0
    move.w	#$0101,d1			turns on mouseDownIRQ
    trap	#15				-
    
    move.w	#$ff00,d1			clears the screen
    move.b	#11,d0				-
    trap	#15				-
    
    move.b	#0,contin			resets contin to 0
    
    move.b  #11,d0              moves cursor to center screen
    move.w  #$0810,d1           -
    trap    #15                 -
    
    move.b	#81,d0          changes background to black
    move.l	#BLACK,d1
    trap	#15 
 
    lea    endMssg,a1       	displays the end message
    move.b	#14,d0
    trap	#15

    repeat                  runs untin mouse is clicked
    	nop
    until.b contin <eq> #1 do
    
    bsr		new_game        calls the new_gane function
    bra		reset           branches to the reset position

    SIMHALT             ; halt simulator


************************************************************************
*Resets everything to prepare for a new game
************************************************************************
new_game:
	clr.l	d0		resets all of the data registers
	clr.l	d1
	clr.l	d3
	clr.l	d4
	clr.l	d5
	clr.l	d6
	clr.w	currentCell
	lea		$20000,a6	resets the cell stack
	move.b	#0,gmeovr		resets gameover
	rts    
   
************************************************************************
*takes the coordinates in d1 and then checks to see if there is a wall 
*in place at that spot in the window
************************************************************************ 
check_wall:
    move.l  -(sp),d0
    bsr     coordSwitch
    move.b  #22,d0
    trap    #15
    
    if.b d1 <eq> #'#' then
        add.b   #1,wall
    endi
    if.b d1 <eq> #'-' then
        add.b   #1,wall
    endi
    if.b d1 <eq> #'|' then
        add.b   #1,wall
    endi
    if.b d1 <eq> #'.' then
        add.b   #1,wall
    endi
    if.b d1 <eq> #'_' then
        add.b   #1,wall
    endi
    move.l  (sp)+,d0
    rts
    
*************************************************************************
*The next 4 subroutines checks currentCell's adjacent cells and tests if 
*they have all of their walls intact. if they do then they set their 
*respective variables equal to 1
*************************************************************************
check_down:
    move.l d1,-(sp)
    move.b  #0,wall
    clr.l   d1
    move.w  currentCell,d1
    add.w   #2,d1       move south of currentCell
    move.w  d1,chkingCell
    
    sub.w   #1,d1       move to top wall of checkingCell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #$100,d1       move to right wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #1,d1           move to bottom wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    sub.w   #$100,d1        move to left wall of checking cell
    bsr     check_wall
    
    if.b wall <eq> #4 then  if all walls are up then set downOK = 1
        move.b  #1,downOK
    else
        move.b  #0,downOK
    endi
    move.l  (sp)+,d1
    rts
    
check_up:
    move.l  d1,-(sp)
    move.b  #0,wall     clears wall variable
    clr.l   d1
    move.w  currentCell,d1
    sub.w   #2,d1       move north of current cell
    move.w  d1,chkingCell
    
    sub.w   #1,d1       move to top wall of checkingCell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #$100,d1       move to right wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #1,d1           move to bottom wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    sub.w   #$100,d1        move to left wall of checking cell
    bsr     check_wall
    
    if.b wall <eq> #4 then  if all walls are up then set upOK = 1
        move.b  #1,upOK
    else
        move.b  #0,upOK
    endi
    move.l  (sp)+,d1
    rts
    
check_left:
    move.l  d1,-(sp)
    move.b  #0,wall     clears wall variable
    clr.l   d1
    move.w  currentCell,d1
    sub.w   #$200,d1       move west of current cell
    move.w  d1,chkingCell
    
    sub.w   #1,d1       move to top wall of checkingCell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #$100,d1       move to right wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #1,d1           move to bottom wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    sub.w   #$100,d1        move to left wall of checking cell
    bsr     check_wall
    
    if.b wall <eq> #4 then  if all walls are up then set leftOK = 1
        move.b  #1,leftOK
    else
        move.b  #0,leftOK
    endi
    move.l  (sp)+,d1
    rts
    
check_right:
    move.l  d1,-(sp)
    move.b  #0,wall     	clears wall variable
    clr.l   d1
    move.w  currentCell,d1
    add.w   #$200,d1       	move east of current cell
    move.w  d1,chkingCell
    
    sub.w   #1,d1       	move to top wall of checkingCell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #$100,d1       	move to right wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    add.w   #1,d1           move to bottom wall of checking cell
    bsr     check_wall
    
    clr.l   d1
    move.w  chkingCell,d1
    sub.w   #$100,d1        move to left wall of checking cell
    bsr     check_wall
    
    if.b wall <eq> #4 then  if all walls are up then set rightOK = 1
        move.b  #1,rightOK
    else
        move.b  #0,rightOK
    endi
    move.l  (sp)+,d1
    rts
    
    
    
    
display_title:
	movem.l d0-d2/d6/a1,-(sp)
	lea		title,a1            loads the title.txt file
	move.b	#51,d0
	trap	#15
	for d6 = #0 to #23 do       displays the file one row at a time
		lea drawTitle,a1
		move.l	#71,d2 
		move.b	#53,d0
		trap	#15

		lea		drawTitle,a1
		move.b	#14,d0
		trap	#15
	endf
	movem.l	(sp)+,d0-d2/d6/a1
	rts   
*************************************************************************
*Generates the maze. first part loads the maze template in from maze.txt
* second part uses a depth-first algorithm to carve out a maze from the 
*template
*************************************************************************    
load_maze:
    movem.l d0-d2/d6/a1,-(sp) 

    move.b  #81,d0          changes background to blue
    move.l  #BACKGROUND,d1        -
    trap    #15             -
   
	lea	maze1,a1            loads the mazeHash.txt file
	move.b	#51,d0          -
	trap	#15             -
	for d6 = #0 to #28 do   displays the file one line at a time
		lea	draw,a1
		move.l	#81,d2
		move.b	#53,d0
		trap	#15
	
		lea	draw,a1
		move.b	#14,d0
		trap	#15
	endf
	bsr     carve_maze          calls maze carving algorithm
	movem.l (sp)+,d0-d2/d6/a1
	rts
	
	
carve_maze:
    movem.l d0-d2,-(sp)
    
    lea     $20000,a6   			creates a cellstack
 *   move.w  #1,totalCells     		468 total cells in the grid
    move.w  #$0101,currentCell     	first cell is in top left
    move.w  #1,visitedCells    		We have been to one cell
    while visitedCells <lt> #507 do
        bsr     check_down      	sets downOK
        bsr     check_up        	sets upOK
        bsr     check_left      	sets leftOK
        bsr     check_right     	sets rightOK
        
        clr.l   d2
        add.b   downOK,d2       	adds all check variables into d2
        add.b   upOK,d2
        add.b   leftOK,d2
        add.b   rightOK,d2
        
        if.b d2 <gt> #0 then 		if there is at least one cell with all walls intact
new_num
            bsr     rand_gen    	puts a random # 0-3 into random
            move.w  currentCell,nextCell
            if.b random <eq> #0 then
                if.b downOK <eq> #1 then
                add.w   #2,nextCell
                move.w  nextCell,d1
                
                sub.w   #1,d1            moves cursor in position to destroy wall
                move.b  #11,d0              -
                trap    #15                 -
                
                move.b  #$20,d1             destroys wall
                move.b  #6,d0               -
                trap    #15                 -
                   
                move.w  nextCell,d1         moves the cursor into newCell
                move.b  #11,d0
                trap    #15
                
                move.w  currentCell,-(a6)   pushes old cell onto cellstack
                move.w  nextCell,currentCell    makes newCell currentCell
                add.w   #1,visitedCells        increments visitedCells
                endi
            else
                if.b random <eq> #1 then
                    if.b leftOK <eq> #1 then
                    sub.w   #$200,nextCell
                    move.w  nextCell,d1
                
                    add.w   #$100,d1            moves cursor in position to destroy wall
                    move.b  #11,d0              -
                    trap    #15                 -
                
                    move.b  #$20,d1             destroys wall
                    move.b  #6,d0               -
                    trap    #15                 -
                   
                    move.w  nextCell,d1         moves the cursor into newCell
                    move.b  #11,d0
                    trap    #15
                
                    move.w  currentCell,-(a6)   pushes old cell onto cellstack
                    move.w  nextCell,currentCell    makes newCell currentCell
                    add.w   #1,visitedCells        increments visitedCells
                    endi
                else
                    if.b random <eq> #2 then
                        if.b upOK <eq> #1 then
                        sub.w   #2,nextCell
                        move.w  nextCell,d1
                
                        add.w   #1,d1            moves cursor in position to destroy wall
                        move.b  #11,d0              -
                        trap    #15                 -
                
                        move.b  #$20,d1             destroys wall
                        move.b  #6,d0               -
                        trap    #15                 -
                   
                        move.w  nextCell,d1         moves the cursor into newCell
                        move.b  #11,d0
                        trap    #15
                
                        move.w  currentCell,-(a6)   pushes old cell onto cellstack
                        move.w  nextCell,currentCell    makes newCell currentCell
                        add.w   #1,visitedCells        increments visitedCells 
                        endi
                    else
                        if.b random <eq> #3 then
                            if.b rightOK <eq> #1 then
                            add.w   #$200,nextCell
                            move.w  nextCell,d1
                
                            sub.w   #$100,d1            moves cursor in position to destroy wall
                            move.b  #11,d0              -
                            trap    #15                 -
                
                            move.b  #$20,d1             destroys wall
                            move.b  #6,d0               -
                            trap    #15                 -
                   
                            move.w  nextCell,d1         moves the cursor into newCell
                            move.b  #11,d0
                            trap    #15
                
                            move.w  currentCell,-(a6)   pushes old cell onto cellstack
                            move.w  nextCell,currentCell    makes newCell currentCell
                            add.w   #1,visitedCells        increments visitedCells 
                            endi
                         else
                            bra new_num			avoids having to go through checking adjacent cells again. More efficient
                         endi
                      endi
                   endi
                endi
            else
                move.w  (a6)+,currentCell       no possible moves so move back to previous cell
            endi
    endw
    
    move.b	#81,d0         		changes background to green
    move.l	#LIME,d1
    trap	#15
    
    move.w	#$4d19,d1		Marks the goal of the maze
    move.b	#11,d0			-
    trap	#15			-
    
    move.b  #$45,d1         		- 	   
    move.b  #6,d0           		-    
    trap    #15 			-
    
    move.b	#81,d0          	changes background to White
    move.l	#WHITE,d1
    trap	#15    
    
    move.w	#$0101,d1		Marks the beginning of the maze
    move.b	#11,d0			-
    trap	#15			-
    
    move.b  #$40,d1         		- 	   
    move.b  #6,d0           		-    
    trap    #15 			-
    
    move.b	#81,d0          	changes background to backgroundcolor
    move.l	#BACKGROUND,d1
    trap	#15  
    
    movem.l (sp)+,d0-d2
    rts
    
    
**********************************************************************
*Takes current time and uses as a seed for the random number generator
**********************************************************************
seed_rand:
    move.l  d0,-(sp)
    move.b	#8,d0
	trap	#15
	bsr     coordSwitch
	clr.l   d4
	clr.l   d5
	move.l	d1,PRNlword(a3)
	move.l  (sp)+,d0
	rts
    
********************************************************************************************
*Generates a random number between 0 and 3 and stores it in random
********************************************************************************************
rand_gen:
    move.l  d0,-(sp)
	bsr	random_n
	bsr     random_n
	move.l	PRNlword(a3),d0
	and.l	#$FFFF,d0
	divu.w	#4,d0
	lsr.l   #8,d0
	lsr.l   #8,d0
	move.b  d0,random
	move.l  (sp)+,d0
	rts

********************************************************************************************
*Random number sequence generator. Code from Lee Davison's Maze Program
********************************************************************************************
random_n:
	MOVEM.l	d0-d2,-(sp)			* save d0, d1 and d2
	MOVE.l	PRNlword(a3),d0		* get current seed longword
	MOVEQ		#$AF-$100,d1		* set EOR value
	MOVEQ		#18,d2			* do this 19 times
Ninc0
	ADD.l		d0,d0				* shift left 1 bit
	BCC.s		Ninc1				* if bit not set skip feedback

	EOR.b		d1,d0				* do Galois LFSR feedback
Ninc1
	DBF		d2,Ninc0			* loop

	MOVE.l	d0,PRNlword(a3)		* save back to seed longword
	MOVEM.l	(sp)+,d0-d2			* restore d0, d1 and d2

	RTS
    

*********************************************************************************************
*Switches the format of the coords in d1 from d1 = 0000clrw to d1 = 00rw00cl
*Needed because task 22 requires a different format for coords than task 11
*********************************************************************************************    
coordSwitch:
    movem.l d4/d5,-(sp)
    move.b  d1,d4
    clr.b   d1
    lsr.l   #8,d1
    lsl.l   #8,d4
    lsl.l   #8,d4
    move.w  d1,d5
    move.l  d4,d1
    move.w  d5,d1
    movem.l (sp)+,d4/d5
    rts
*********************************************************************************************
moveClr:
    move.b	#81,d0          changes background to background color
    move.l	#BACKGROUND,d1
    trap	#15  
    
    move.b  #$20,d1 erases previous character
    move.b  #6,d0   -
    trap    #15     -        
        
    move.w  d3,d1   moves cursor to new coord
    move.b  #11,d0  -
    trap    #15     -  
    
    move.b	#81,d0          changes background to white
    move.l	#WHITE,d1
    trap	#15 
 
    move.b  #$40,d1 displays the user character
    move.b  #6,d0   -
    trap    #15     -   

    move.w  d3,d1   moves cursor back to the appropriate place after the previous write
    move.b  #11,d0  -
    trap    #15     - 
    rts
*********************************************************************************************



keyDownIRQ
    movem.l d0-d5,-(sp)
    move.l  #$26282527,d1   up,down,left,right
    move.b  #19,d0
    trap    #15
**************************************************************
    btst.l  #31,d1  test if up key was pressed
    if <ne> then
    	clr.l	d1
        move.w  #$00ff,d1  gets current cursor position
        move.b  #11,d0  -
        trap    #15     -
        move.w  d1,d2   stores cursor position into d2
        
        sub.w   #1,d1   moves coord up one
        move.w  d1,d3   saves new coord into d3
        
        *test for invalid move task 22
        bsr coordSwitch
        move.b  #22,d0
        trap    #15
        if.b d1 <eq> #'#' then
            nop
        else
            if.b d1 <eq> #'E' then
                move.b  #1,gmeovr
                bsr     moveClr
            else
                bsr     moveClr
            endi
        endi
        
***************************************************************************************************  
    else
        btst.l  #21,d1  test if down key was pressed
        if <ne> then
        	clr.l	d1			needed but I have no clue why
            move.w  #$00ff,d1  gets current cursor position
            move.b  #11,d0  -
            trap    #15     -
            move.w  d1,d2   stores cursor position into d2
        
            add.w   #1,d1   moves coord down one
            move.w  d1,d3   saves the new coord
        
            *test for invalid move here using task 22
            bsr coordSwitch
            move.b  #22,d0
            trap    #15
            if.b d1 <eq> #'#' then
                nop
            else
                if.b d1 <eq> #'E' then
                    move.b  #1,gmeovr
                    bsr     moveClr
                else
                    bsr     moveClr
                endi
            endi

*****************************************************************************************************       
        else
            btst.l  #15,d1      test if left key was pressed
            if <ne> then
            	******
            	*move.b	#1,gmeovr          USED FOR TESTING PURPOSES ONLY
            	******
            	clr.l	d1
                move.w  #$00ff,d1       gets current cursor position
                move.b  #11,d0          -
                trap    #15             -
                move.w  d1,d2           stores cursor position into d2
        
                move.b  d1,d4           saves the row coord 
                sub.w   #$100,d1        moves coord left one
                move.b  d4,d1           moves the saved row coord back in
                move.w  d1,d3           saves new coord into d2
        
                *test for invalid move task 22
                bsr coordSwitch
                move.b  #22,d0
                trap    #15
                if.b d1 <eq> #'#' then
                    nop
                else
                    if.b d1 <eq> #'E' then
                        move.b  #1,gmeovr
                        bsr     moveClr
                    else
                        bsr     moveClr
                    endi
                endi

******************************************************************************************************        
            else
                btst.l  #7,d1   test if right key was pressed
                if <ne> then
                	clr.l	d1
                    move.w  #$00ff,d1       gets current cursor position
                    move.b  #11,d0          -
                    trap    #15             -
                    move.w  d1,d2           stores cursor position into d2
        
                    move.b  d1,d4           saves the row coord 
                    add.w   #$100,d1         moves coord right one
                    move.b  d4,d1           moves the saved row coord back in
                    move.w  d1,d3           saves new coord into d2
        
                    *test for invalid move task 22
                    bsr coordSwitch
                    move.b  #22,d0
                    trap    #15
                    if.b d1 <eq> #'#' then
                        nop
                    else
                        if.b d1 <eq> #'E' then
                            move.b  #1,gmeovr
                            bsr     moveClr
                        else
                            bsr     moveClr
                        endi
                    endi
                endi
            endi
        endi
    endi
                 
    movem.l (sp)+,d0-d5
    rte
    
    
********************************************************************
mouseDownIRQ
	movem.l	d0-d1,-(sp)
	move.b	#11,d0
	move.w	#$ff00,d1
	trap	#15
	
	move.w	#$0000,d1
	trap	#15
	
	move.b	#1,contin
	movem.l	(sp)+,d0-d1
	rte
***********************************************************************************************************    
*VARS
gmeovr          ds.b    1
lose		ds.b	1
PRNlword	ds.l	1
random          ds.b    1
totalCells      ds.w    1
visitedCells    ds.w    1
currentCell     ds.w    1
chkingCell      ds.w    1
nextCell        ds.w    1
wall            ds.b    1
changeCell	ds.w	1
rowCell         ds.w    1
saveCell	ds.w	1


downOK          ds.b    1
leftOK          ds.b    1
upOK            ds.b    1
rightOK         ds.b    1

contin		ds.b	1

    
*TEXT
endMssg		dc.b	'CONGRATS!  YOU CONQUERED THE MAZE.  '
            	dc.b    'CLICK ANYWHERE TO PLAY AGAIN...',0

*MAZE INFO
maze1		dc.b	'mazeHash.txt',0
draw		ds.b	81
		dc.b	0
title		dc.b	'title.txt',0
drawTitle 	ds.b	71
		dc.b	0

    END    START        ; last line of source

















*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~8~
